#!/usr/bin/python
import sys, time, os, argparse, logging
import solver, lowspectro_splitter, fuckit_splitter, demo_downloader
from parse_solver_args import parse_solver_options,print_solver_help_and_exit
from selenium import webdriver

parser = argparse.ArgumentParser(description="Runs the complete captcha solving process with different values for the number of lowest captchas to find, in order to find the best value")

#Solver options
parser.add_argument('-s', nargs='*', action='append', help='Adds a solver to the list of solvers to be used to solve the captcha. ' 
                    + 'This option can be specified multiple times to use additional solvers as backup.  Use -sh for information on specifying solvers')
parser.add_argument('-sh', required=False, action='store_true', help='Show the syntax to specify solvers')

#Misc args
parser.add_argument('-fu', required=False, action='store_true', help='Use the fuckit splitter')
parser.add_argument('-l', type=int, required=False, default=sys.maxint, metavar='limit', help='The number of captchas to solve')
parser.add_argument('-proxy_host', required=False, default="", type=str, metavar='proxy_host', help='Use the specified proxy host')
parser.add_argument('-proxy_port', required=False, default="8118", type=str, metavar='proxy_host', help='Use the specified proxy port')
parser.add_argument('-w', type=int, required=False, default=0, metavar='seconds', help='The number of seconds to wait between rounds')
parser.add_argument('-v', required=False, action='store_true', help='Verbose mode')
parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode')

args = parser.parse_args()
if args.sh:
    print_solver_help_and_exit()

output_level = logging.CRITICAL
if args.vv:
    output_level = logging.DEBUG
elif args.v:
    output_level = logging.INFO

masher = None 
downloader =  demo_downloader.DemoDownloader(args.proxy_host, args.proxy_port, debug = output_level)
if args.fu:
    splitter = fuckit_splitter.FuckItSplitter(debug = output_level) 
else:
    splitter = lowspectro_splitter.LowSpectroSplitter(debug = output_level)
solver_list, solver_thresholds = parse_solver_options(args.s, output_level)
captcha_solver = solver.CaptchaSolver(solver_list, splitter, [downloader], solver_thresholds, masher, output_level)

start = os.getpid()
correct = 0
streak = 0
combo = 0
i = start

fp = webdriver.FirefoxProfile()
if args.proxy_host != "":
    fp.set_preference("network.proxy.type", 1)# Direct = 0, Manual = 1, PAC = 2, AUTODETECT = 4, SYSTEM = 5
    fp.set_preference("network.proxy.socks", args.proxy_host)
    fp.set_preference("network.proxy.socks_port", int(args.proxy_port))
    fp.set_preference("network.proxy.no_proxies_on", "") # set this value as desired 
else:
    fp.set_preference("network.proxy.type", 0)# Direct = 0, Manual = 1, PAC = 2, AUTODETECT = 4, SYSTEM = 5
driver = webdriver.Firefox(firefox_profile=fp)
driver.get("https://www.google.com/recaptcha/demo/")
while i <  start + args.l:
    filename = "current" + str(start)

    error = 1
    url = ""
    while error != 0 and error < 5:
        try:
            driver.execute_script("javascript:Recaptcha.switch_type('audio');")
            error = 0
        except:
            error += 1
            print "Got ", error - 1, "errors"
            time.sleep(error)
    
    if error == 0:
        href = None
        while href == None:  #Wait for the href if selenium returns before the link has appeared
            try:
                href = driver.find_element_by_id("recaptcha_audio_download")
            except:
                print "Couldn't find audio download link"

        url = str(href.get_attribute("href"))
    
    if url == "":
        print "Could not get url"
        continue

    print "Downloading..."
    if not downloader.get_mp3(url, filename):
        print "Could not download mp3 file",filename + ".mp3"
        continue
    print "Done downloading"

    #Solve the captcha word by word
    answer = captcha_solver.solve(filename)

    if answer == "":
        print "No answer found for",filename
        success = False
    else:
        print "Submitting " + answer 
        captcha_textbox = None
        while captcha_textbox == None:
            try:
                captcha_textbox = driver.find_element_by_id("recaptcha_response_field")
            except:
                print "Couldn't find captcha textbox"
                time.sleep(.05)

        if captcha_textbox == None:
            print "Couldn't find captcha textbox"
            continue

        captcha_textbox.send_keys(answer)
        captcha_inputs = driver.find_elements_by_tag_name("input")
        for element in captcha_inputs:
            try:
                if element.get_attribute("value") == "Submit":
                    element.click()
                    print "Clicked Submit"
            except selenium.common.exceptions.StaleElementReferenceException:
                continue
        print "Waiting for selenium"
        success = driver.find_elements_by_tag_name('form')[0].text.find("Incorrect") == -1
        captcha_solver.delete_files(filename)

    if success:
        correct += 1
        print "Correct ",
        streak += 1
        print "(" + str(streak) + " in a row)",
        if streak > combo:
            combo = streak
    else:
        print "Incorrect ",
        streak = 0

    print "(Highscore " + str(combo) + ")",
    print correct,"/",i - start + 1," = ",round(float(correct)/float(i - start + 1) * 100, 2),"%"

    time.sleep(args.w)
    i += 1

