import Image, os, subprocess, operator, logging, argparse, time, sys, math
from splitterbase import SplitterBase

class FuckItSplitter(SplitterBase):
    """Wraps all the methods required to split an audio recaptcha into the words using the pitch"""
    def __init__(self, debug = logging.CRITICAL, debug_images = False, filename_prefix = "temp/", filename_suffix = ".mp3",
                    noisered_file = "", number_of_words = 5):
        self.logger = logging.getLogger('FuckItSplitter')
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)
        self.debug_images = debug_images

        self.noisered_file = noisered_file
        self.filename_prefix = filename_prefix
        self.filename_suffix = filename_suffix
        self.image_index = {}
        self.number_of_words = number_of_words

    def split_captcha(self, filename, split_components = [ "audio", "image" ]):
        """Splits a audio captcha and its pitch.  If only an audio file or only a spectrogram
            image file is needed for each word, split_components can be used to only select the component you need."""  
        fullfile = self.filename_prefix + filename + self.filename_suffix
        duration = self.get_duration(fullfile, True)

        words = []
        length = duration/self.number_of_words
        for i in range(0, self.number_of_words):
            words.append([length * i, length * (i+1)])
        self.logger.debug("Found %d words: %s", len(words), str(words))

        if "audio" in split_components:
            if self.noisered_file != "":
                self.remove_noise(fullfile, self.noisered_file, extension = self.filename_suffix)
            self.split_wav_file(words, 0, filename, columnx_unit = "second", soundsizex = duration) 
        if "image" in split_components:
            self.image_index = {} #Reset the file nameing convention
            self.get_spectrogram(filename)
            spectrogram = self.filename_prefix + filename + ".png"
            image = Image.open(spectrogram).convert("RGB")
            image,box = self.remove_white_border(image)
            if self.debug_images:
                image.save(self.filename_prefix + filename + "_noborder.png", "PNG")
            sizex, sizey = image.size

            for i in range(0, len(words)):
                cropped_image = image.crop((int(words[i][0]/duration * sizex), 0, int(words[i][1]/duration * sizex), sizey))
                cropped_image.save(self.filename_prefix + filename + "_" + str(i) + ".png", "PNG")

        return len(words)


if __name__ == "__main__":    
    parser = argparse.ArgumentParser(description="Split an audio captcha into the individual words.  For the most of these options, you can mostly leave the defaults unless you feel like experimenting")
    parser.add_argument('filename', help='The audio file to split into individual words')
    parser.add_argument('-v', required=False, action='store_true', help='Verbose mode')
    parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode')

    args = parser.parse_args()
    output_level = logging.CRITICAL
    debug_images = False
    if args.vv:
        output_level = logging.DEBUG
        debug_images = True
    elif args.v:
        output_level = logging.INFO

    dirname, filename = os.path.split(args.filename)
    if dirname != "":
        dirname += "/"
    extension = filename.split(".")[-1]
    filename = filename[:-1-len(extension)]

    splitter = FuckItSplitter(debug = output_level, debug_images = debug_images,filename_prefix = dirname, 
                                     filename_suffix = "." + extension)
    splitter.split_captcha(filename)



